#!/usr/bin/env python

import click

@click.group()
def app():
    pass

__all__ = [
    'app',
]

from .commands import create
from .commands import clone
from .commands import fork
from .commands import remote
from .commands import pullrequest


if __name__ == '__main__':
    app()



import bs4

import fedora.client

from pag.utils import repo_url

class PagureException(Exception):
    pass


class Pagure(fedora.client.OpenIdBaseClient):
    def __init__(self, url='https://pagure.io', insecure=False):
        super(Pagure, self).__init__(
            base_url=url,
            login_url=url + "/login/",
            useragent="pag (cli)",
            debug=False,
            insecure=insecure,
            openid_insecure=insecure,
            username=None,  # We supply this later
            cache_session=True,
            retries=7,
            timeout=120,
            retry_backoff_factor=0.3,
        )

    @property
    def is_logged_in(self):
        response = self._session.get(self.base_url)
        return "logout" in response.text

    def create(self, name, description):
        if not self.is_logged_in:
            raise PagureException('Not logged in.')
        url = self.base_url + '/new'
        response = self._session.get(url)
        if not bool(response):
            raise PagureException("Couldn't get form to get "
                                  "csrf token %r" % response)

        soup = bs4.BeautifulSoup(response.text, "html.parser")
        data = dict(
            csrf_token=soup.find(id='csrf_token').attrs['value'],
            name=name,
            description=description,
        )

        response = self._session.post(url, data=data)

        if not bool(response):
            del data['csrf_token']
            raise PagureException('Bad status code from pagure when '
                                  'creating project: %r.  Sent %r' % (
                                      response, data))
        return repo_url(name)

    def fork(self, name):
        if not self.is_logged_in:
            raise PagureException('Not logged in.')

        url = self.base_url + '/' + name
        response = self._session.get(url)
        if not bool(response):
            raise PagureException("Couldn't get form to get "
                                  "csrf token %r" % response)

        soup = bs4.BeautifulSoup(response.text, "html.parser")
        data = dict(
            csrf_token=soup.find(id='csrf_token').attrs['value'],
        )

        url = self.base_url + '/do_fork/' + name
        response = self._session.post(url, data=data)

        if not bool(response):
            del data['csrf_token']
            raise PagureException('Bad status code from pagure when '
                                  'forking project: %r.  Sent %r' % (
                                      response, data))
        return repo_url(name)

    def submit_pull_request(self, name, base, head, title, comment):
        url = 'https://pagure.io/{name}/diff/{base}..{head}'
        url = url.format(name=name, base=base, head=head)

        response = self._session.get(url)
        if not bool(response):
            raise PagureException("Couldn't get form to get "
                                  "csrf token %r" % response)

        soup = bs4.BeautifulSoup(response.text, "html.parser")
        data = dict(
            csrf_token=soup.find(id='csrf_token').attrs['value'],
            branch_to=base,
            title=title,
            initial_comment=comment,
        )
        response = self._session.post(url, data=data)

        if not bool(response):
            del data['csrf_token']
            raise PagureException('Bad status code from pagure when '
                                  'creating pull request: %r.  Sent %r' % (
                                      response, data))

        return response.url


client = Pagure()



import getpass

import click

from pag.app import app
from pag.utils import (
    configured,
    in_git_repo,
    repo_url,
    run,
)
from pag.client import client


@app.command()
@click.argument('name')
@click.argument('description')
@click.confirmation_option(prompt="Are you sure you want to create a new repo?")
@configured
def create(conf, name, description):

    click.echo("Trying to create %r in pagure.io" % name)
    if not client.is_logged_in:
        password = getpass.getpass("FAS password for %r" % conf['username'])
        client.login(username=conf['username'], password=password)
    url = client.create(name, description)
    click.echo("Created %s" % url)

    local_repo = in_git_repo()
    if local_repo is None or local_repo != name:
        url = repo_url(name, ssh=True, git=True)
        run(['git', 'clone', url, name.split('/')[-1]])
    else:
        url = repo_url(name, ssh=True, git=True)
        name = name.split('/')[0]
        run(['git', 'remote', 'add', name, url])
        run(['git', 'remote', 'add', 'origin', url])



import getpass
import sys

import click

from pag.app import app
from pag.utils import (
    configured,
    assert_local_repo,
    in_git_repo,
    get_default_upstream_branch,
    get_current_local_branch,
    run,
    die,
)
from pag.client import client

HEADER = "Pull request title goes here."
MARKER = "# All lines below this marker are ignored."


def split_input(branch, default_repo):
    tokens = branch.split(':')
    if len(tokens) > 2:
        die("%r is a malformed repo:branch expression." % branch)
    elif len(tokens) == 1:
        repo, branch = default_repo, tokens[0]
    elif len(tokens) == 2:
        repo, branch = tokens
        repo = repo + '/' + default_repo
    else:
        raise RuntimeError('Should not be possible to get here...')
    return repo, branch


@app.command('pull-request')
@assert_local_repo
@click.option('-b', '--base')
@click.option('-h', '--head')
@configured
def pullrequest(conf, base, head):

    name = in_git_repo()

    if base is None:
        try:
            base = get_default_upstream_branch(name)
        except Exception:
            click.echo("Failed to find default upstream branch for %r" % name)
            click.echo("Please specify a base branch explicitly.")
            sys.exit(1)
    else:
        name, base = split_input(base, name)

    if head is None:
        head = get_current_local_branch()
    else:
        name, head = split_input(head, name)
        if '/' in name:
            name = 'fork/' + name

    cmd = ['git', 'log', '{base}..{head}'.format(base=base, head=head)]
    _, log = run(cmd, echo=False)

    def modify(line):
        if not line:
            return line
        if line[0].isspace():
            return line.strip()
        return '# ' + line

    log = '\n'.join([modify(line) for line in log.split('\n')])
    edited = click.edit(
        "\n\n".join([HEADER, MARKER, log]),
        env=dict(VIMINIT='set filetype="gitcommit"'),
    )
    if not edited.strip():
        click.echo("Aborting due to empty pull request message.")
        sys.exit(1)
    title, comment = edited.split('\n', 1)
    if not title:
        click.echo("Aborting due to empty pull request message.")
        sys.exit(1)
    comment = comment.split(MARKER)[0]
    comment = comment.strip()

    username = conf['username']
    if not client.is_logged_in:
        password = getpass.getpass("FAS password for %r" % username)
        client.login(username=username, password=password)
    url = client.submit_pull_request(name, base, head, title, comment)
    click.echo(url)



import functools
import os
import subprocess as sp
import sys

import click
import requests
import yaml


CONF_FILE = os.path.expanduser('~/.config/pag')


def run(cmd, echo=True, graceful=True):
    click.echo('  $ ' + " ".join(cmd))
    proc = sp.Popen(cmd, stdout=sp.PIPE, stderr=sp.STDOUT)
    output, _ = proc.communicate()
    output = output.decode('utf-8')
    if echo:
        click.echo(output)
    if not graceful and proc.returncode != 0:
        sys.exit(1)
    return proc.returncode, output


def die(msg, code=1):
    click.echo(msg)
    sys.exit(code)


def in_git_repo():
    # TODO -- would be smarter to look "up" the tree, too.
    if not os.path.exists('.git') or not os.path.isdir('.git'):
        return None
    return os.getcwd().split('/')[-1]


def assert_local_repo(func):
    @functools.wraps(func)
    def inner(*args, **kwargs):
        if not in_git_repo():
            die("fatal:  Not a git repository")
        return func(*args, **kwargs)
    return inner


def get_default_upstream_branch(name):
    url = 'https://pagure.io/api/0/projects'
    response = requests.get(url, params=dict(pattern=name, fork=False))
    if not bool(response):
        raise IOError("Failed to talk to %r %r", (url, response))
    data = response.json()
    projects = data['projects']
    if not projects:
        raise ValueError("No such project %r" % name)
    if len(projects) > 1:
        raise ValueError("More than one project called %r found "
                         "(%i of them, in fact)." % (name, len(projects)))
    project = projects[0]
    return project['default_branch']


def get_current_local_branch():
    code, stdout = run(['git', 'branch', '--contains'])
    return stdout.split(maxsplit=1)[1].strip()


def repo_url(name, ssh=False, git=False, domain='pagure.io'):
    if ssh:
        prefix = 'ssh://git@'
    else:
        prefix = 'https://'

    if '/' in name:
        if git:
            suffix = 'forks/%s' % name
        else:
            suffix = 'fork/%s' % name
    else:
        suffix = '%s' % name

    if git:
        suffix = suffix + '.git'

    return prefix + domain + '/' + suffix


def create_config():
    username = input("FAS username:  ")
    conf = dict(
        username=username,
    )

    with open(CONF_FILE, 'wb') as f:
        f.write(yaml.dump(conf).encode('utf-8'))

    click.echo("Wrote %r" % CONF_FILE)


def load_config():
    with open(CONF_FILE, 'rb') as f:
        return yaml.load(f.read().decode('utf-8'))

def load_or_create_config():
    if not os.path.exists(CONF_FILE):
        click.echo("%r not found.  Creating..." % CONF_FILE)
        create_config()
    return load_config()


def configured(func):
    @functools.wraps(func)
    def inner(*args, **kwargs):
        config = load_or_create_config()
        return func(config, *args, **kwargs)
    return inner



import click
import requests

from pag.app import app
from pag.utils import (
    assert_local_repo,
    in_git_repo,
    repo_url,
    die,
    run
)


@app.group()
@assert_local_repo
def remote():
    pass

@remote.command()
@click.argument('name')
def add(name):
    repo = in_git_repo()
    url = repo_url(name + '/' + repo)
    response = requests.head(url)
    if not bool(response):
        die("No such url %s, %r" % (url, response))

    repo = in_git_repo()
    url = repo_url(name + '/' + repo, ssh=True, git=True)
    name = name.split('/')[0]
    return run(['git', 'remote', 'add', name, url])



import getpass

import click

from pag.app import app
from pag.utils import (
    configured,
    assert_local_repo,
    in_git_repo,
    repo_url,
    run,
)
from pag.client import client


@app.command()
@assert_local_repo
@configured
def fork(conf):
    username = conf['username']
    name = in_git_repo()

    click.echo("Trying to fork %r in pagure.io" % name)
    if not client.is_logged_in:
        password = getpass.getpass("FAS password for %r" % username)
        client.login(username=username, password=password)
    url = client.fork(name)
    click.echo("Created %s" % url)

    name = username + '/' + name
    url = repo_url(name, ssh=True, git=True)
    name = name.split('/')[0]
    run(['git', 'remote', 'add', name, url])



#!/usr/bin/env python
from setuptools import setup, find_packages

#import sys
#import os
#
#f = open('README.rst')
#long_description = f.read().strip()
#long_description = long_description.split('split here', 1)[1]
#f.close()


long_description = ''
requires = []

version = '0.6'

setup(
    name='pag',
    version=version,
    description="Commandline interaction with pagure.io",
    long_description=long_description,
    author='Ralph Bean',
    author_email='rbean@redhat.com',
    url='https://pagure.io/pag',
    license='GPLv3+',
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3 :: Only",
    ],
    install_requires=requires,
    test_suite='nose.collector',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    entry_points="""
    [console_scripts]
    pag = pag.app:app
    pagcli = pag.app:app
    """
)



import click

from pag.app import app
from pag.utils import (
    repo_url,
    run,
)

@app.command()
@click.argument('name')
def clone(name):
    url = repo_url(name, ssh=True, git=True)
    run(['git', 'clone', url, name.split('/')[-1]])
